@@ -7,7 +7,7 @@
 /** @internal return cache, or throw lua error if not open */
 static struct kr_cache * cache_assert_open(lua_State *L)
 {
-	struct kr_cache *cache = &the_worker->engine->resolver.cache;
+	struct kr_cache *cache = &the_resolver->cache;
 	if (kr_fails_assert(cache) || !kr_cache_is_open(cache))
 		lua_error_p(L, "no cache is open yet, use cache.open() or cache.size, etc.");
 	return cache;
@@ -16,12 +16,10 @@ static struct kr_cache * cache_assert_open(lua_State *L)
 /** Return available cached backends. */
 static int cache_backends(lua_State *L)
 {
-	struct engine *engine = the_worker->engine;
-
 	lua_newtable(L);
-	for (unsigned i = 0; i < engine->backends.len; ++i) {
-		const struct kr_cdb_api *api = engine->backends.at[i];
-		lua_pushboolean(L, api == engine->resolver.cache.api);
+	for (unsigned i = 0; i < the_engine->backends.len; ++i) {
+		const struct kr_cdb_api *api = the_engine->backends.at[i];
+		lua_pushboolean(L, api == the_resolver->cache.api);
 		lua_setfield(L, -2, api->name);
 	}
 	return 1;
@@ -99,16 +97,16 @@ static int cache_stats(lua_State *L)
 	return 1;
 }
 
-static const struct kr_cdb_api *cache_select(struct engine *engine, const char **conf)
+static const struct kr_cdb_api *cache_select(const char **conf)
 {
 	/* Return default backend */
 	if (*conf == NULL || !strstr(*conf, "://")) {
-		return engine->backends.at[0];
+		return the_engine->backends.at[0];
 	}
 
 	/* Find storage backend from config prefix */
-	for (unsigned i = 0; i < engine->backends.len; ++i) {
-		const struct kr_cdb_api *api = engine->backends.at[i];
+	for (unsigned i = 0; i < the_engine->backends.len; ++i) {
+		const struct kr_cdb_api *api = the_engine->backends.at[i];
 		if (strncmp(*conf, api->name, strlen(api->name)) == 0) {
 			*conf += strlen(api->name) + strlen("://");
 			return api;
@@ -170,8 +168,6 @@ static int cache_open(lua_State *L)
 		lua_error_p(L, "expected 'open(number max_size, string config = \"\")'");
 
 	/* Select cache storage backend */
-	struct engine *engine = the_worker->engine;
-
 	lua_Integer csize_lua = lua_tointeger(L, 1);
 	if (!(csize_lua >= 8192 && csize_lua < SIZE_MAX)) { /* min. is basically arbitrary */
 		lua_error_p(L, "invalid cache size specified, it must be in range <8192, "
@@ -181,19 +177,20 @@ static int cache_open(lua_State *L)
 
 	const char *conf = n > 1 ? lua_tostring(L, 2) : NULL;
 	const char *uri = conf;
-	const struct kr_cdb_api *api = cache_select(engine, &conf);
+	const struct kr_cdb_api *api = cache_select(&conf);
 	if (!api)
 		lua_error_p(L, "unsupported cache backend");
 
 	/* Close if already open */
-	kr_cache_close(&engine->resolver.cache);
+	kr_cache_close(&the_resolver->cache);
 
 	/* Reopen cache */
 	struct kr_cdb_opts opts = {
-		(conf && strlen(conf)) ? conf : ".",
-		cache_size
+		.is_cache = true,
+		.path = (conf && strlen(conf)) ? conf : ".",
+		.maxsize = cache_size,
 	};
-	int ret = kr_cache_open(&engine->resolver.cache, api, &opts, engine->pool);
+	int ret = kr_cache_open(&the_resolver->cache, api, &opts, &the_engine->pool);
 	if (ret != 0) {
 		char cwd[PATH_MAX];
 		get_workdir(cwd, sizeof(cwd));
@@ -202,7 +199,7 @@ static int cache_open(lua_State *L)
 	}
 	/* Let's check_health() every five seconds to avoid keeping old cache alive
 	 * even in case of not having any work to do. */
-	ret = kr_cache_check_health(&engine->resolver.cache, 5000);
+	ret = kr_cache_check_health(&the_resolver->cache, 5000);
 	if (ret != 0) {
 		kr_log_error(CACHE, "periodic health check failed (ignored): %s\n",
 				kr_strerror(ret));
@@ -224,7 +221,7 @@ static int cache_open(lua_State *L)
 
 static int cache_close(lua_State *L)
 {
-	struct kr_cache *cache = &the_worker->engine->resolver.cache;
+	struct kr_cache *cache = &the_resolver->cache;
 	if (!kr_cache_is_open(cache)) {
 		return 0;
 	}
@@ -264,8 +261,7 @@ static int cache_clear_everything(lua_State *L)
 	lua_error_maybe(L, ret);
 
 	/* Clear reputation tables */
-	struct kr_context *ctx = &the_worker->engine->resolver;
-	lru_reset(ctx->cache_cookie);
+	lru_reset(the_resolver->cache_cookie);
 	lua_pushboolean(L, true);
 	return 1;
 }
@@ -336,12 +332,10 @@ static int cache_get(lua_State *L)
  * in NS elections again. */
 static int cache_ns_tout(lua_State *L)
 {
-	struct kr_context *ctx = &the_worker->engine->resolver;
-
 	/* Check parameters */
 	int n = lua_gettop(L);
 	if (n < 1) {
-		lua_pushinteger(L, ctx->cache_rtt_tout_retry_interval);
+		lua_pushinteger(L, the_resolver->cache_rtt_tout_retry_interval);
 		return 1;
 	}
 
@@ -354,8 +348,8 @@ static int cache_ns_tout(lua_State *L)
 				STR(UINT_MAX));
 	}
 
-	ctx->cache_rtt_tout_retry_interval = interval_lua;
-	lua_pushinteger(L, ctx->cache_rtt_tout_retry_interval);
+	the_resolver->cache_rtt_tout_retry_interval = interval_lua;
+	lua_pushinteger(L, the_resolver->cache_rtt_tout_retry_interval);
 	return 1;
 }
 
