@@ -53,10 +53,17 @@ struct __attribute((packed)) kr_sockaddr_un_key {
 
 extern inline uint64_t kr_rand_bytes(unsigned int size);
 
+extern inline const char * kr_strerror(int e);
+
 /* Logging & debugging */
 bool kr_dbg_assertion_abort = DBG_ASSERTION_ABORT;
 int kr_dbg_assertion_fork = DBG_ASSERTION_FORK;
 
+
+// Non-inline instance to allow warn-free use of kr_assert() etc. in other `extern inline`.
+extern inline bool kr_assert_func(bool result, const char *expr, const char *func,
+				  const char *file, int line);
+
 void kr_fail(bool is_fatal, const char *expr, const char *func, const char *file, int line)
 {
 	const int errno_orig = errno;
@@ -739,14 +746,14 @@ void kr_bitmask(unsigned char *a, size_t a_len, int bits)
 		a[i] = 0;
 }
 
-int kr_rrkey(char *key, uint16_t class, const knot_dname_t *owner,
+int kr_rrkey(char *key, uint16_t rrclass, const knot_dname_t *owner,
 	     uint16_t type, uint16_t additional)
 {
 	if (!key || !owner) {
 		return kr_error(EINVAL);
 	}
 	uint8_t *key_buf = (uint8_t *)key;
-	int ret = u16tostr(key_buf, class);
+	int ret = u16tostr(key_buf, rrclass);
 	if (ret <= 0) {
 		return ret;
 	}
