@@ -100,8 +100,8 @@ KR_EXPORT KR_COLD void kr_fail(bool is_fatal, const char* expr, const char *func
 
 /** Use kr_require(), kr_assert() or kr_fails_assert() instead of directly this function. */
 __attribute__ ((warn_unused_result))
-static inline bool kr_assert_func(bool result, const char *expr, const char *func,
-				  const char *file, int line)
+KR_EXPORT inline bool kr_assert_func(bool result, const char *expr, const char *func,
+					const char *file, int line)
 {
 	if (!result)
 		kr_fail(false, expr, func, file, line);
@@ -170,9 +170,11 @@ typedef struct kr_http_header_array_entry {
 /** Array of HTTP headers for DoH. */
 typedef array_t(kr_http_header_array_entry_t) kr_http_header_array_t;
 
-/** Concatenate N strings. */
+/** Concatenate N strings and put the result into a mempool. */
 KR_EXPORT
-char* kr_strcatdup(unsigned n, ...);
+char* kr_strcatdup_pool(knot_mm_t *pool, unsigned n, ...);
+/** Concatenate N strings. */
+#define kr_strcatdup(n, ...) kr_strcatdup_pool(NULL, n, ## __VA_ARGS__)
 
 /** Construct absolute file path, without resolving symlinks.
  * \return malloc-ed string or NULL (+errno in that case) */
@@ -309,7 +311,7 @@ struct sockaddr *kr_sockaddr_from_key(struct sockaddr_storage *dst,
 KR_EXPORT
 bool kr_sockaddr_key_same_addr(const char *key_a, const char *key_b);
 
-/** Compare two given sockaddr.
+/** Compare two given sockaddr.  (only address and port)
  * return 0 - addresses are equal, error code otherwise.
  */
 KR_EXPORT KR_PURE
@@ -351,6 +353,9 @@ int kr_family_len(int family);
  * Also accepts IPv6 link-local and AF_UNIX starting with "/" (ignoring port) */
 KR_EXPORT
 struct sockaddr * kr_straddr_socket(const char *addr, int port, knot_mm_t *pool);
+/** Like kr_straddr_socket() but use `sa` instead of allocating. */
+KR_EXPORT
+struct sockaddr * kr_straddr_socket_set(struct sockaddr *sa, const char *addr, int port);
 
 /** Parse address and return subnet length (bits).
   * @warning 'dst' must be at least `sizeof(struct in6_addr)` long. */
@@ -367,7 +372,11 @@ int kr_straddr_subnet(void *dst, const char *addr);
  * \note Only internet addresses are supported, i.e. no AF_UNIX sockets.
  */
 KR_EXPORT
-int kr_straddr_split(const char *instr, char ipaddr[static restrict (INET6_ADDRSTRLEN + 1)],
+int kr_straddr_split(const char *instr, char ipaddr[
+			#ifndef __cplusplus
+			static restrict (INET6_ADDRSTRLEN + 1)
+			#endif
+		     ],
 		     uint16_t *port);
 
 /** Formats ip address and port in "addr#port" format.
@@ -417,7 +426,7 @@ static inline bool kr_sockaddr_link_local(const struct sockaddr *addr)
   * @return key length if successful or an error
   * */
 KR_EXPORT
-int kr_rrkey(char *key, uint16_t class, const knot_dname_t *owner,
+int kr_rrkey(char *key, uint16_t rrclass, const knot_dname_t *owner,
 	     uint16_t type, uint16_t additional);
 
 /** Add RRSet copy to a ranked RR array.
