@@ -23,6 +23,7 @@ struct kr_zonecut {
 	struct kr_zonecut *parent; /**< Parent zone cut. */
 	trie_t *nsset;        /**< Map of nameserver => address_set (pack_t). */
 	knot_mm_t *pool;     /**< Memory pool. */
+	bool avoid_resolving; /**< Avoid resolving the NS names. */
 };
 
 /**
@@ -152,7 +153,7 @@ int kr_zonecut_set_sbelt(struct kr_context *ctx, struct kr_zonecut *cut);
 KR_EXPORT
 int kr_zonecut_find_cached(struct kr_context *ctx, struct kr_zonecut *cut,
 			   const knot_dname_t *name, const struct kr_query *qry,
-			   bool * restrict secured);
+			   bool *secured);
 /**
  * Check if any address is present in the zone cut.
  *
