@@ -218,6 +218,13 @@ function policy.FLAGS(opts_set, opts_clear)
 	end
 end
 
+-- Set price-factor
+function policy.PRICE_FACTOR16(factor)
+	return function(_, req)
+		ffi.C.defer_set_price_factor16(req, factor)
+	end
+end
+
 local function mkauth_soa(answer, dname, mname, ttl)
 	if mname == nil then
 		mname = dname
@@ -857,10 +864,14 @@ function policy.TAGS_ASSIGN(names)
 end
 
 -- Perform a list of actions sequentially; meant for kr_view_insert_action().
+-- Return value of the last one is propagated.
 function policy.COMBINE(list)
 	if #list == 1 then return list[1] end
 	local r = 'function(state,req) '
-		for _, item in ipairs(list) do
+		for i, item in ipairs(list) do
+			if i == #list then
+				r = r .. 'return '
+			end
 			r = r .. item .. '(state,req); '
 		end
 	return r .. 'end'
@@ -934,9 +945,15 @@ policy.layer = {
 
 		if ffi.C.kr_view_select_action(req, view_action_buf) == 0 then
 			local act_str = ffi.string(view_action_buf[0].data, view_action_buf[0].len)
-			return loadstring('return '..act_str)()(state, req)
+			local new_state = loadstring('return '..act_str)()(state, req)
+			-- We still respect the chain-rule notion, i.e. we skip
+			-- lua-configured policy rules iff the action was "final"
+			-- (`refused` and `noanswer` in the current 6.x)
+			if new_state ~= nil then return new_state end
 		end
 
+		if ffi.C.ratelimiting_request_begin(req) then return end
+
 		local qry = req:initial() -- same as :current() but more descriptive
 		return policy.evaluate(policy.rules, req, qry, state)
 			or state
